package ru.dbotthepony.kstarbound.defs.actor

import com.google.common.collect.ImmutableList
import com.google.gson.JsonElement
import com.google.gson.JsonNull
import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.defs.AssetPath
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.util.random.nextRange
import ru.dbotthepony.kstarbound.util.random.random
import java.util.random.RandomGenerator
import kotlin.math.max

data class NPCVariant(
	val species: Registry.Entry<Species>,
	val scripts: ImmutableList<AssetPath>,
	val initialScriptDelta: Int = 5,
) {
	@JsonFactory
	data class SerializedData(
		val levelVariance: Vector2d = Vector2d.ZERO,
		val scripts: ImmutableList<AssetPath>,
		val initialScriptDelta: Int = 5,
		val scriptConfig: JsonElement = JsonNull.INSTANCE,
		val humanoidConfig: String? = null,
	)

	companion object {
		suspend fun create(species: Registry.Entry<Species>, type: String, level: Double, random: RandomGenerator, overrides: JsonElement = JsonNull.INSTANCE): NPCVariant {
			val config = Registries.buildNPCConfig(type, overrides)

			val serialized = Starbound.gson.fromJson(config, SerializedData::class.java)
			val finalLevel = max(0.0, random.nextRange(serialized.levelVariance) + level)
			TODO()
		}

		suspend fun create(species: Registry.Entry<Species>, type: String, level: Double, seed: Long, overrides: JsonElement = JsonNull.INSTANCE): NPCVariant {
			return create(species, type, level, random(seed), overrides)
		}
	}
}
