package ru.dbotthepony.kstarbound.lua.bindings

import ru.dbotthepony.kstarbound.math.vector.Vector2d
import ru.dbotthepony.kstarbound.lua.LuaEnvironment
import ru.dbotthepony.kstarbound.lua.from
import ru.dbotthepony.kstarbound.lua.get
import ru.dbotthepony.kstarbound.lua.luaFunction
import ru.dbotthepony.kstarbound.lua.set
import ru.dbotthepony.kstarbound.lua.toByteString
import ru.dbotthepony.kstarbound.math.Line2d
import ru.dbotthepony.kstarbound.world.entities.AbstractEntity
import ru.dbotthepony.kstarbound.world.entities.ActorEntity
import ru.dbotthepony.kstarbound.world.entities.MonsterEntity
import ru.dbotthepony.kstarbound.world.entities.player.PlayerEntity
import ru.dbotthepony.kstarbound.world.entities.tile.WorldObject

fun provideEntityBindings(self: AbstractEntity, lua: LuaEnvironment) {
	if (self is WorldObject)
		provideWorldObjectBindings(self, lua)

	if (self is MonsterEntity)
		provideMonsterBindings(self, lua)

	if (self is ActorEntity)
		provideStatusControllerBindings(self.statusController, lua)

	provideWorldBindings(self.world, lua)

	val table = lua.newTable()
	lua.globals["entity"] = table

	table["id"] = luaFunction { returnBuffer.setTo(self.entityID.toLong()) }
	table["position"] = luaFunction { returnBuffer.setTo(from(self.position)) }
	table["entityType"] = luaFunction { returnBuffer.setTo(self.type.jsonName.toByteString()) }
	table["uniqueId"] = luaFunction { returnBuffer.setTo(self.uniqueID.get().toByteString()) }
	table["persistent"] = luaFunction { returnBuffer.setTo(self.isPersistent) }

	table["entityInSight"] = luaFunction { target: Number ->
		val entity = self.world.entities[target.toInt()] ?: return@luaFunction returnBuffer.setTo(false)
		returnBuffer.setTo(self.world.chunkMap.collide(Line2d(self.position, entity.position)) { it.type.isSolidCollision } == null)
	}

	table["isValidTarget"] = luaFunction { target: Number ->
		val entity = self.world.entities[target.toInt()] ?: return@luaFunction returnBuffer.setTo(false)

		if (!self.team.get().canDamage(entity.team.get(), self == entity)) // original engine always passes `false` to `isSelfDamage`
			return@luaFunction returnBuffer.setTo(false)

		if (entity is MonsterEntity)
			return@luaFunction returnBuffer.setTo(entity.isAggressive)

		// TODO: NPC handling here

		returnBuffer.setTo(entity is PlayerEntity)
	}

	table["damageTeam"] = luaFunction {
		val result = newTable()

		result["team"] = self.team.get().team.toLong()
		result["type"] = self.type.jsonName.toByteString()

		returnBuffer.setTo(result)
	}

	table["distanceToEntity"] = luaFunction { entity: Number ->
		val find = self.world.entities[entity.toInt()]

		if (find != null) {
			returnBuffer.setTo(from(self.world.geometry.diff(find.position, self.position)))
		} else {
			returnBuffer.setTo(from(Vector2d.ZERO))
		}
	}
}
