package ru.dbotthepony.kstarbound.world.entities.behavior

import com.google.common.collect.ImmutableList
import org.classdump.luna.runtime.ExecutionContext
import ru.dbotthepony.kstarbound.lua.userdata.BehaviorState

class DynamicNode(val children: ImmutableList<AbstractBehaviorNode>) : AbstractBehaviorNode() {
	private var index = 0

	override fun run(delta: Double, state: BehaviorState): Status {
		for ((i, node) in children.withIndex()) {
			val status = node.runAndReset(delta, state)

			if (status == Status.FAILURE && index == i)
				index++
			else if (i < index && (status == Status.SUCCESS || status == Status.FAILURE)) {
				node.reset()
				index = i
			}

			if (status == Status.SUCCESS || index >= children.size) {
				return status
			}
		}

		return Status.RUNNING
	}

	override fun toString(): String {
		return "DynamicNode[${children.size}, index=$index, current=${children.getOrNull(index)}]"
	}

	override fun reset() {
		children.forEach { it.reset() }
		index = 0
	}
}
